import { __decorate, __param } from 'vscode/external/tslib/tslib.es6.js';
import { Disposable } from 'vscode/vscode/vs/base/common/lifecycle';
import { IAccessibilityService } from 'vscode/vscode/vs/platform/accessibility/common/accessibility.service';
import { ILogService } from 'vscode/vscode/vs/platform/log/common/log.service';
import { IDebugService } from 'vscode/vscode/vs/workbench/contrib/debug/common/debug.service';

let ReplAccessibilityAnnouncer = class ReplAccessibilityAnnouncer extends Disposable {
    static { this.ID = 'debug.replAccessibilityAnnouncer'; }
    constructor(debugService, accessibilityService, logService) {
        super();
        const viewModel = debugService.getViewModel();
        this._register(viewModel.onDidFocusSession((session) => {
            if (!session) {
                return;
            }
            this._register(session.onDidChangeReplElements((element) => {
                if (!element || !('originalExpression' in element)) {
                    return;
                }
                const value = ( element.toString());
                accessibilityService.status(value);
                logService.trace('ReplAccessibilityAnnouncer#onDidChangeReplElements', element.originalExpression + ': ' + value);
            }));
        }));
    }
};
ReplAccessibilityAnnouncer = ( __decorate([
    ( __param(0, IDebugService)),
    ( __param(1, IAccessibilityService)),
    ( __param(2, ILogService))
], ReplAccessibilityAnnouncer));

export { ReplAccessibilityAnnouncer };
